#![no_std]
#![no_main]

use esp_hal::uart::{self, DataBits, Parity, StopBits, UartRx};
use esp_hal::usb_serial_jtag::UsbSerialJtag;
use esp_hal::{main, Async};

use panic_halt as _;

const BAUD_RATE: u32 = 250_000;

const MAX_SLOTS: usize = 513;

const START_CODE: u8 = 0;

type Packet = [u8; MAX_SLOTS];

#[main]
fn main() -> ! {
    let peripherals = esp_hal::init(esp_hal::Config::default());

    let mut rx = UartRx::new(
        peripherals.UART0,
        uart::Config::default()
            .with_baudrate(BAUD_RATE)
            .with_data_bits(DataBits::_8)
            .with_parity(Parity::None)
            .with_stop_bits(StopBits::_2)
            .with_rx_timeout(1), // slots
    )
    .unwrap()
    .with_rx(peripherals.GPIO3)
    .into_async();

    let mut usb_serial = UsbSerialJtag::new(peripherals.USB_DEVICE);

    // ignore first (possibly incomplete) packet
    let _ = read_packet(&mut rx).unwrap();

    loop {
        if let Ok(packet) = read_packet(&mut rx) {
            if packet[0] == START_CODE {
                let _ = usb_serial.write_bytes(&packet[1..4]);
            }
        }
    }
}

fn read_packet(rx: &mut UartRx<'_, Async>) -> Result<Packet, uart::Error> {
    let mut buf = [0; MAX_SLOTS];
    let mut pos = 0;

    loop {
        match rx.read_buffered_bytes(&mut buf[pos..]) {
            Ok(bytes_read) => {
                if bytes_read == 0 {
                    break;
                }

                pos += bytes_read;
            }
            Err(err) => return Err(err),
        }
    }

    Ok(buf)
}
